#include <iostream>
#include "poisson.h"
const int Size = 200;

const double Min = 0.0, Max = 1.0;

int main() {
    omp_set_num_threads(2); 
	__cilkrts_set_param("nworkers","2");
    // prepare problem
    Problem p(Size, Min, Max);
    std :: cout << "Problem prepared" << std :: endl;
    // time measurement
    double time;

    // 0. Serial Gauss-Seidel solver
	/*
	SerialSeidelSolver serial;
    serial.SetProblem(&p);
    time = serial.RunAndMeasure();
    std :: cout << "Serial Gauss-Seidel, time = " << time << std :: endl;
	*/

	// 1. Serial Jakobi solver	
	// p.ReinitProblem();
    SerialJakobiSolver serial2;
    serial2.SetProblem(&p);
    time = serial2.RunAndMeasure();
    std :: cout << "Serial Jakobi, time = " << time << std :: endl;
	
	// 2. Parallel Jakobi solver with OpenMP
	/*
    p.ReinitProblem();
    OpenMPJakobiSolver omp_j;
    omp_j.SetProblem(&p);
    time = omp_j.RunAndMeasure();
    std :: cout << "OpenMP Jakobi, time = " << time << std :: endl;
	*/

	// 3. Parallel Jakobi solver with Cilk
	/*
    p.ReinitProblem();
    CilkJakobiSolver cilk_j;
    cilk_j.SetProblem(&p);
    time = cilk_j.RunAndMeasure();
    std :: cout << "Cilk Jakobi, time = " << time << std :: endl;
	*/

	return 0;
}
